// Created by Luca Marceca
// (c) - valenzelektron.com


/*
*	            _                    _      _    _                   
*	           | |                  | |    | |  | |                  
*   __   ____ _| | ___ _ __  _______| | ___| | _| |_ _ __ ___  _ __  
*   \ \ / / _` | |/ _ \ '_ \|_  / _ \ |/ _ \ |/ / __| '__/ _ \| '_ \ 
*    \ V / (_| | |  __/ | | |/ /  __/ |  __/   <| |_| | | (_) | | | |
*     \_/ \__,_|_|\___|_| |_/___\___|_|\___|_|\_\\__|_|  \___/|_| |_|
*  
*   valenzelektron (c) - valenzelektron.com
*
*   @author: Luca Marceca
*   @title: Neural Network Tutorial
*   @programmingLanguage: JavaScript 
*   @commentLanguage: en
*   @version: 1.1.1
*   @update: 13-08-2016 [DD-MM-YYYY]
*	@updateBy: Luca Marceca (LuM)
* 	@updateDescription: Bug fixes
*/

"use strict";


// config
const /* int */ numberQuantity = 3, /* double */ trainingQuantity = 10000;
var weights = [], numbers = [], adjustments = [];
var sum = 0, loggedError = 0, loggedAnswer = 0, output = 0; 

var sigmoidFunction = (s) => { return 1 / (1 + Math.E**(-1 * (s))); }; // normalize inputs in dependence to the weighted sum between 0 and 1
var errorFuntion = (err, num, out, iteration) => { return err * num[iteration] * out * (1 - out); }; // diffrence between the calculated output and the real output; its used to change the weights and actually train the net


/* export */ class NeuralNetwork {
    constructor() {
        this.numbers = numbers;
        this.weights = weights;
        this.output = output;
        this.adjustments = adjustments;

        this.loggedAnswer = loggedAnswer;
        this.loggedError = loggedError;

        this.sum = sum;

        // set initial weights
        for(var j = 0; j < numberQuantity; j++) {
            this.weights.push((Math.random() * 10) - 5); // set a big range
        }
    }

    get errorCalculation() {
        this.sum = 0; // reset

        for(var i = 0; i < this.numbers.length; i++) {
            this.sum += this.weights[i] * this.numbers[i]; // one variant => calculate them together for the main e function
        }

        this.output = sigmoidFunction(this.sum); // calculate diffrence with the std exp. function
        this.loggedError = this.loggedAnswer - this.output; 
        
        // log it public
        output = this.output;
        weights = this.weights;
        numbers = this.numbers;
    }

    get training() {
        // set initial weights
        for(var a = 0; a < numberQuantity; a++) {
            this.weights.push((Math.random() * 10) - 5); // set a big range
        }
        for(var j = 0; j < trainingQuantity; j++) {
            // set random numbers for the first setup
            for(var i = 0; i < numberQuantity; i++) {
                this.numbers.push(Math.floor(Math.random() * 2)); // set a small range
            }

            // define the neural network rule
            if(this.numbers[0] === 0 && this.numbers[1] === 0) this.loggedAnswer = 1;
  
            this.sum = 0; // reset
            
            for(var c = 0; c < this.numbers.length; c++) {   
                this.sum += this.weights[c] * this.numbers[c]; // one variant => calculate them together for the main e function   
            }

            this.output = sigmoidFunction(this.sum); // calculate diffrence with the std exp. function
            this.loggedError = this.loggedAnswer - this.output; 

            if(this.output !== 0.5) {
                for(var d = 0; d < this.numbers.length; d++) {
                    this.adjustments.push(errorFuntion(this.loggedError, this.numbers, this.output, d)); // calculate all together
                }
        
                for(var o = 0; o < this.numbers.length; o++) {
                    this.weights[o] += this.adjustments[o]; // log the new calculated weights
                }
            } // redefine weights
            
            // log it public
            output = this.output;
            weights = this.weights;
            numbers = this.numbers;

            // reset
            this.numbers.length = 0;
            this.adjustments.length = 0;
            this.loggedAnswer = 0;
            
            // debugger;
        }
    }
}

/* export */ function main() {
    // get the input fields
    for(var i = 0; i < numberQuantity; i++) {
        numbers[i] = parseInt(document.getElementById("number" + (i + 1)).value);
    }
    
    // evaluate the inputs
    if(isNaN(numbers[0]) || isNaN(numbers[1]) || isNaN(numbers[2])) { /* ... */ } 
    else { 
        neuralNetwork.errorCalculation; 

        if(output <= 0.75) { // <= 0.7
            console.log("right ans: 0")
        }
        else {
            console.log("right ans: 1")
        }
    }

    // reset
    output = 0;
    loggedAnswer = 0;
    sum = 0;
}

// application start config
var neuralNetwork = new NeuralNetwork(); // generate new neural network
window.onload = function() {
    neuralNetwork.training; // train network
}